@@ -1,8 +1,11 @@
 """
 Helper classes and functions for adapters
 """
+from __future__ import division
 import datetime
+import time
 import locale
+import math
 import numpy
 import pkg_resources
 
@@ -45,7 +48,7 @@ ADAPTER_ENTRY_POINT = 'lizard_map.adapter_class'
 
 LEGEND_WIDTH = 200
 LEFT_LABEL_WIDTH = 100
-BOTTOM_LINE_HEIGHT = FONT_SIZE * 1.5
+BOTTOM_LINE_HEIGHT = FONT_SIZE * 2
 
 
 def _inches_from_pixels(pixels):
@@ -380,11 +383,13 @@ class Graph(object):
                  start_date, end_date,
                  width=None, height=None,
                  today=datetime.datetime.now(),
-                 restrict_to_month=None):
-        self.restrict_to_month = restrict_to_month
+                 restrict_to_month=None,
+                 tz=None):
         self.start_date = start_date
         self.end_date = end_date
         self.today = today
+        self.restrict_to_month = restrict_to_month
+        self.tz = tz
 
         self.figure = Figure()
         if width is None or not width:
@@ -410,12 +415,22 @@ class Graph(object):
         self.axes = self.figure.add_subplot(111)
         self.axes.grid(True)
 
+        # We track whether y_lim has been set manually
+        self._y_min_set_manually = False
+        self._y_max_set_manually = False
+
         # Fixup_axes in init, so axes can be customised (for example set_ylim).
         self.fixup_axes()
 
         #deze kan je zelf zetten
         self.ax2 = None
 
+    def set_ylim(self, y_min, y_max, min_manual=False, max_manual=False):
+        logger.debug('set_ylim y_min = %f y_max = %f' % (y_min, y_max))
+        self.axes.set_ylim(y_min, y_max)
+        self._y_min_set_manually = min_manual
+        self._y_max_set_manually = max_manual
+
     def add_today(self):
         # Show line for today.
         self.axes.axvline(self.today, color='orange', lw=1, ls='--')
@@ -450,12 +465,33 @@ class Graph(object):
                     self.end_date.replace(tzinfo=data[0][0].tzinfo)
             index_in_daterange = ((data[0] < end_date_tz) &
                                   (data[0] > start_date_tz))
+
+            # Calculate correct y_min and y_max, but use old if they have
+            # already been set manually
             if index_in_daterange.any():
                 data_low = numpy.min(data[1, index_in_daterange])
                 data_high = numpy.max(data[1, index_in_daterange])
                 data_span = data_high - data_low
+
                 view_low = data_low - data_span * bottom
                 view_high = data_high + data_span * top
+
+                # Don't zoom in too much if values are essentially the same
+                # and differ only in noise. Values shown at the Y-axis should
+                # only have 2 decimals.
+                view_low = math.floor(view_low * 40) / 40
+                view_high = math.ceil(view_high * 40) / 40
+                while (view_high - view_low) < 0.03:
+                    # Difference is only 0.025 (or 0!), differences of
+                    # smaller than 0.01 show up at the y-axis.
+                    view_low -= 1.0 / 80
+                    view_high += 1.0 / 80
+
+                if self._y_min_set_manually:
+                    view_low, _ = self.axes.get_ylim()
+                if self._y_max_set_manually:
+                    _, view_high = self.axes.get_ylim()
+
                 self.axes.set_ylim(view_low, view_high)
         return None
 
@@ -488,7 +524,7 @@ class Graph(object):
             axes_to_change.xaxis.set_major_locator(major_locator)
 
             major_formatter = MultilineAutoDateFormatter(
-                major_locator, axes_to_change)
+                major_locator, axes_to_change, tz=self.tz)
             axes_to_change.xaxis.set_major_formatter(major_formatter)
 
         available_height = (self.height -
@@ -578,9 +614,9 @@ class Graph(object):
                 ncol=ncol,
                 fancybox=True,
                 shadow=True,)
-         #legend.set_size('medium')
-         # TODO: get rid of the border around the legend.
-         # to get rid of the border: graph.axes.legend_.draw_frame(False)
+            #legend.set_size('medium')
+            # TODO: get rid of the border around the legend.
+            # to get rid of the border: graph.axes.legend_.draw_frame(False)
 
     def init_second_axes(self):
         """ init second axes """
@@ -625,3 +661,159 @@ class Graph(object):
         response = HttpResponse(content_type='image/png')
         canvas.print_png(response)
         return response
+
+    def render(self):
+        '''
+        more general alias for http_png(), to support FlotGraph
+        should return a valid HttpResponse
+        '''
+        return self.http_png()
+
+class FlotGraphAxes(object):
+    legend_ = None
+
+    def __init__(self):
+        self.flot_data = [] # list of dicts in the format {'label': x, 'data':[(x, y), (x, y)]}
+
+    def set_ylabel(self, ylabel):
+        self.ylabel = ylabel
+
+    def axhline(
+        self,
+        ybase,
+        color=None,
+        lw=None,
+        ls=None,
+        label=None
+    ):
+        '''no-op for FlotGraph'''
+        pass
+
+    def plot(
+        self,
+        xvalues,
+        yvalues,
+        lw=None,
+        color=None,
+        label=None
+    ):
+        # convert xvalues to timestamps for flot.js
+        xvalues = [float(time.mktime(x.timetuple()) * 1000) for x in xvalues]
+        self.flot_data.append({
+            'label': label,
+            'data': zip(xvalues, yvalues),
+            'color': color,
+            'lines': {'show': True}
+        })
+
+    def bar(
+        self,
+        xvalues,
+        yvalues,
+        edgecolor='blue',
+        width=0,
+        label=None
+    ):
+        # convert xvalues to timestamps for flot.js
+        xvalues = [float(time.mktime(x.timetuple()) * 1000) for x in xvalues]
+        self.flot_data.append({
+            'label': label,
+            'data': zip(xvalues, yvalues),
+            'color': edgecolor,
+            'bars': {'show': True, 'barWidth': width, 'align': 'center'}
+        })
+
+    def grid(self, grid):
+        '''no-op for FlotGraph'''
+        pass
+
+    def get_ylim(self):
+        '''no-op for FlotGraph'''
+        return None, None
+
+class FlotGraph(object):
+    """
+    Class for flot graphs, i.e. for popups, krw graphs
+
+    - horizontal axis = dates
+    - vertical axis = user defined
+    - outputs a dict (which in turn gets converted to Json) conforming to the client-side
+      JavaScript code which uses flot.js
+    """
+
+    def __init__(
+        self,
+        start_date, end_date,
+        today=datetime.datetime.now(),
+        restrict_to_month=None,
+        tz=None
+    ):
+        self.start_date = start_date
+        self.end_date = end_date
+        self.today = today
+        self.restrict_to_month = restrict_to_month
+        self.tz = tz
+
+        self.axes = FlotGraphAxes()
+        self.responseobject = None # unused
+        self.xlabel = None
+        self.ylabel = None
+
+    def set_ylim(self, y_min, y_max, min_manual=False, max_manual=False):
+        self.y_min = y_min
+        self.y_max = y_max
+        self.min_manual = min_manual
+        self.max_manual = max_manual
+
+    def add_today(self):
+        '''no-op for FlotGraph'''
+        '''should add a vertical line for the current day?'''
+        pass
+
+    def set_xlabel(self, xlabel):
+        self.xlabel = xlabel
+
+    def set_ylabel(self, ylabel):
+        self.ylabel = ylabel
+
+    def suptitle(self, suptitle):
+        self.suptitle = suptitle
+
+    def fixup_axes(self, second=False):
+        '''no-op for FlotGraph'''
+        pass
+
+    def legend_space(self):
+        '''no-op for FlotGraph'''
+        pass
+
+    def legend(
+       self,
+       handles=None,
+       labels=None,
+       ncol=1,
+       force_legend_below=False
+    ):
+        '''no-op for FlotGraph'''
+        pass
+
+    def init_second_axes(self):
+        '''no-op for FlotGraph'''
+        pass
+
+    def get_bar_width(self, delta_t):
+        '''assumes xvalues are always dates'''
+        # seconds = delta_t.total_seconds()  # 2.7 only.
+        seconds = (delta_t.microseconds +
+                   (delta_t.seconds + delta_t.days * 24 * 3600) * 10**6) / 10**6
+        return float(seconds * 1000)
+
+    def http_png(self):
+        raise NotImplementedError('not implemented for a FlotGraph, perhaps you meant to use Graph?')
+
+    def render(self):
+        return {
+            'data': self.axes.flot_data,
+            'x_label': self.xlabel,
+            'y_label': self.ylabel
+        }
