@@ -2,7 +2,6 @@ import datetime
 from django import forms
 from django.conf import settings
 
-from datewidget import SelectDateWidget
 from django.forms.widgets import RadioSelect
 
 from lizard_map.daterange import PERIOD_CHOICES
@@ -11,12 +10,15 @@ from lizard_map.dateperiods import MONTH
 from lizard_map.models import StatisticsMixin
 from lizard_map.models import WorkspaceStorage
 
+import logging
+logger = logging.getLogger(__name__)
+
 
 class WorkspaceSaveForm(forms.Form):
     """
     Save workspace from edit to storage
     """
-    name = forms.CharField(max_length=100, required=True, label='naam')
+    name = forms.CharField(max_length=100, required=True, label='Naam')
 
 
 class WorkspaceLoadForm(forms.Form):
@@ -68,12 +70,12 @@ class DateRangeForm(forms.Form):
         label='',)
     # TODO: NL date format.  Also hardcoded in the js.
     dt_start = forms.DateTimeField(
-        label='van',
-        widget=SelectDateWidget(years=years_choices),
+        label='Van',
+        widget=forms.DateInput(attrs={'class': 'datepicker'}),
         required=False)
     dt_end = forms.DateTimeField(
-        label='t/m',
-        widget=SelectDateWidget(years=years_choices),
+        label='Tot',
+        widget=forms.DateInput(attrs={'class': 'datepicker'}),
         required=False)
 
     def __init__(self, *args, **kwargs):
@@ -84,8 +86,8 @@ class DateRangeForm(forms.Form):
 
         # Set initial dt_start/end on disabled when not selected.
         if args and 'period' in args[0] and args[0]['period'] != PERIOD_OTHER:
-            self.fields['dt_start'].widget.attrs['disabled'] = True
-            self.fields['dt_end'].widget.attrs['disabled'] = True
+            self.fields['dt_start'].widget.attrs['disabled'] = "disabled"
+            self.fields['dt_end'].widget.attrs['disabled'] = "disabled"
 
 
 class CollageForm(forms.Form):
@@ -93,6 +95,10 @@ class CollageForm(forms.Form):
     Collage form. Never actually displayed.
     """
     workspace_id = forms.IntegerField(required=True)
+
+    # Values should be "workspace_storage" or "workspace_edit", but this
+    # is not enforced. If no value is given, assume "workspace_edit".
+    workspace_type = forms.CharField(max_length=20, required=False)
     x = forms.FloatField(required=True)
     y = forms.FloatField(required=True)
     radius = forms.FloatField(required=True)
@@ -103,10 +109,16 @@ class CollageAddForm(forms.Form):
     """
     Collage form. Never actually displayed.
     """
-    name = forms.CharField(max_length=100, required=True)
-    adapter_class = forms.CharField(max_length=100, required=True)
-    adapter_layer_json = forms.CharField(max_length=100, required=True)
-    identifier = forms.CharField(max_length=100, required=True)
+
+    # Name in the database is max 80 characters, but I still removed
+    # the max_length because the view takes care of it already, and
+    # clicks shouldn't silently fail.
+    name = forms.CharField(required=True)
+
+    # The others don't need a max_length either.
+    adapter_class = forms.CharField(required=True)
+    adapter_layer_json = forms.CharField(required=True)
+    identifier = forms.CharField(required=True)
 
 
 class EmptyForm(forms.Form):
@@ -162,6 +174,7 @@ class CollageItemEditorForm(forms.Form):
         """
         """
         super(CollageItemEditorForm, self).__init__(*args, **kwargs)
+
         # Leave out week and day [:4].
         self.fields['aggregation_period'].choices = (
             StatisticsMixin.AGGREGATION_PERIOD_CHOICES[:4])
@@ -180,6 +193,9 @@ class CollageItemEditorForm(forms.Form):
             (11, 'alleen november'),
             (12, 'alleen december'),
             )
+
         # Initial status
-        if int(kwargs['initial']['aggregation_period']) != MONTH:
+        # Sometimes this has no 'aggregation_period', for some reason
+        if int(kwargs.get('initial', dict()).
+               get('aggregation_period', '-1')) != MONTH:
             self.fields['restrict_to_month'].widget.attrs['disabled'] = True
