@@ -1,48 +1,39 @@
-# Unchecked imports start here
+import datetime
 import logging
-import mapnik
+import random
+import string
 
 from django.contrib.auth.models import User
 from django.core.cache import cache
 from django.core.urlresolvers import reverse
 from django.db import models
-from django.db.models.signals import post_save
 from django.db.models.signals import post_delete
+from django.db.models.signals import post_save
 from django.utils import simplejson as json
 from django.utils.translation import ugettext as _
-from south.modelsinspector import add_introspection_rules
+import jsonfield
+import lizard_map.configchecker
+import mapnik
 import pkg_resources
 
-from lizard_map.dateperiods import ALL
-from lizard_map.dateperiods import YEAR
-from lizard_map.dateperiods import QUARTER
-from lizard_map.dateperiods import MONTH
-from lizard_map.dateperiods import WEEK
-from lizard_map.dateperiods import DAY
-from lizard_map.dateperiods import calc_aggregation_periods
-from lizard_map.dateperiods import fancy_period
+from lizard_map import dateperiods
+from lizard_map import fields
+from lizard_map.adapter import AdapterClassNotFoundError
+from lizard_map.adapter import adapter_class_names
+from lizard_map.adapter import adapter_entrypoint
+from lizard_map.adapter import adapter_layer_arguments
+from lizard_map.adapter import adapter_serialize
+from lizard_map.exceptions import WorkspaceItemError
 from lizard_map.mapnik_helper import point_rule
-# Unchecked end here
+# Temporary, because fewsjdbc api handler imports this.
+from lizard_map.adapter import ADAPTER_ENTRY_POINT
 
-import lizard_map.configchecker
+ADAPTER_ENTRY_POINT  # Pyflakes...
 lizard_map.configchecker  # Pyflakes...
 
-from adapter import adapter_layer_arguments
-from adapter import adapter_entrypoint
-from adapter import adapter_class_names
-from adapter import adapter_serialize
-
-# Temporary, because fewsjdbc api handler imports this.
-from adapter import ADAPTER_ENTRY_POINT
-ADAPTER_ENTRY_POINT
-
-# New imports
-import datetime
-from django.core.serializers.json import DjangoJSONEncoder
-
-from lizard_map.adapter import AdapterClassNotFoundError
-#from jsonfield.fields import JSONField
-
+# workspace storage's secret slugs
+SECRET_SLUG_CHARS = string.ascii_lowercase
+SECRET_SLUG_LENGTH = 8
 
 # Do not change the following items!
 GROUPING_HINT = 'grouping_hint'
@@ -62,181 +53,6 @@ LOCATION_ENTRY_POINT = 'lizard_map.location_method'
 ADAPTER_CLASS_WMS = 'wms'
 
 logger = logging.getLogger(__name__)
-# Add introspection rules for ColorField
-add_introspection_rules([], ["lizard_map.models.ColorField"])
-
-
-def legend_values(min_value, max_value, min_color, max_color, steps):
-    """Interpolates colors between min_value and max_value, calc
-    corresponding colors and gives boundary values for each band.
-
-    Makes list of dictionaries: {'color': Color, 'low_value':
-    low value, 'high_value': high value}"""
-    result = []
-    value_per_step = (max_value - min_value) / steps
-    for step in range(steps):
-        try:
-            fraction = float(step) / (steps - 1)
-        except ZeroDivisionError:
-            fraction = 0
-        alpha = (min_color.a * (1 - fraction) +
-                 max_color.a * fraction)
-        red = (min_color.r * (1 - fraction) +
-               max_color.r * fraction)
-        green = (min_color.g * (1 - fraction) +
-                 max_color.g * fraction)
-        blue = (min_color.b * (1 - fraction) +
-                max_color.b * fraction)
-        color = Color('%02x%02x%02x%02x' % (red, green, blue, alpha))
-
-        low_value = min_value + step * value_per_step
-        high_value = min_value + (step + 1) * value_per_step
-        result.append({
-                'color': color,
-                'low_value': low_value,
-                'high_value': high_value,
-                })
-    return result
-
-
-class Color(str):
-    """Simple color object: r, g, b, a.
-
-    The object is in fact a string with class variables.
-    """
-
-    def __init__(self, s):
-        self.r = None
-        self.g = None
-        self.b = None
-        if s is None:
-            return
-        try:
-            self.r = int(s[0:2], 16)
-        except ValueError:
-            self.r = 128
-        try:
-            self.g = int(s[2:4], 16)
-        except ValueError:
-            self.b = 128
-        try:
-            self.b = int(s[4:6], 16)
-        except ValueError:
-            self.b = 128
-        try:
-            # Alpha is optional.
-            self.a = int(s[6:8], 16)
-        except ValueError:
-            self.a = 255
-
-    def to_tuple(self):
-        """
-        Returns color values in a tuple. Values are 0..1
-        """
-        result = (self.r / 255.0, self.g / 255.0,
-                  self.b / 255.0, self.a / 255.0)
-        return result
-
-    @property
-    def html(self):
-        """
-        Returns color in html format.
-        """
-        if self.r is not None and self.g is not None and self.b is not None:
-            return '#%02x%02x%02x' % (self.r, self.g, self.b)
-        else:
-            return '#ff0000'  # Red as alarm color
-
-
-class ColorField(models.CharField):
-    """Custom ColorField for use in Django models. It's an extension
-    of CharField."""
-
-    default_error_messages = {
-        'invalid': _(
-            u'Enter a valid color code rrggbbaa, '
-            'where aa is optional.'),
-        }
-    description = "Color representation in rgb"
-
-    # Ensures that to_python is always called.
-    __metaclass__ = models.SubfieldBase
-
-    def __init__(self, *args, **kwargs):
-        kwargs['max_length'] = 8
-        super(ColorField, self).__init__(*args, **kwargs)
-
-    def get_prep_value(self, value):
-        return str(value)
-
-    def to_python(self, value):
-        if isinstance(value, Color):
-            return value
-        return Color(value)
-
-
-#From:
-#https://github.com/bradjasper/django-jsonfield/blob/master/jsonfield/fields.py
-#Using djang-jsonfield results in an error in admin pages using this
-#field.
-
-# Add south introspection rules.
-try:
-    from south.modelsinspector import add_introspection_rules
-    add_introspection_rules([], ["^lizard_map\.models\.JSONField"])
-except:
-    # South is not used.
-    pass
-
-
-class JSONField(models.TextField):
-    """JSONField is a generic textfield that neatly serializes/unserializes
-JSON objects seamlessly"""
-
-    # Used so to_python() is called
-    __metaclass__ = models.SubfieldBase
-
-    def to_python(self, value):
-        """Convert our string value to JSON after we load it from the DB"""
-
-        if value == "":
-            return None
-
-        try:
-            if isinstance(value, basestring):
-                return json.loads(value)
-        except ValueError:
-            pass
-
-        return value
-
-    def get_db_prep_save(self, value, connection):
-        """Convert our JSON object to a string before we save"""
-
-        if not value or value == "":
-            return None
-
-        if isinstance(value, (dict, list)):
-            value = json.dumps(value, cls=DjangoJSONEncoder)
-
-        # Changed connection to kwarg to fix error.
-        return super(JSONField, self).get_db_prep_save(
-            value, connection=connection)
-
-
-class WorkspaceItemError(Exception):
-    """To be raised when a WorkspaceItem is out of date.
-
-    A WorkspaceItem can represent something that does no longer exist.
-    For example, it may refer to a shape that has been deleted from
-    the database. This error may trigger deletion of such orphans.
-    """
-    pass
-
-
-###### Models start here ######
-
-#### L3 models ####
 
 
 class PeriodMixin(models.Model):
@@ -387,7 +203,7 @@ class WorkspaceItemMixin(models.Model):
                 "Deleting problematic WorkspaceItem: %s", self)
                 # Trac #2470. Return a NullAdapter instead?
             if self.id is not None:
-                # Only delete if it is not saved in the first place.
+                # Only delete if it is saved in the first place.
                 self.delete()
             return None
         return current_adapter
@@ -439,6 +255,29 @@ class WorkspaceItemMixin(models.Model):
         url += '?' + '&'.join(self._url_arguments(identifiers))
         return url
 
+    def has_extent(self):
+        """Return true if this workspace item's adapter can
+        successfully compute an extent and doesn't have "None" in the
+        extent's values."""
+        if not hasattr(self, 'adapter') or not hasattr(self.adapter, 'extent'):
+            return False
+
+        try:
+            extent = self.adapter.extent()
+        except:
+            # Even if the hard disk is in fact on fire, we don't want to raise
+            # an exception here because that may break the whole page.
+            logger.exception("Error when calling .extent()")
+            return False
+
+        if extent is None:
+            return False
+
+        if not isinstance(extent, dict):
+            return False
+
+        return None not in extent.values()
+
 
 class UserSessionMixin(models.Model):
     """For objects that are bound to a user and/or session.
@@ -526,7 +365,7 @@ class WorkspaceModelMixin(object):
         return [to_template_data(workspace_item)
                 for workspace_item in self.workspace_items.all()
                 if workspace_item.adapter_class == ADAPTER_CLASS_WMS
-                and workspace_item.visible ]
+                and workspace_item.visible]
 
     @property
     def is_animatable(self):
@@ -560,14 +399,23 @@ class WorkspaceEdit(
         """
         workspace_storage, _ = WorkspaceStorage.objects.get_or_create(
             name=name, owner=owner)
+
         # Delete all existing workspace item storages.
         workspace_storage.workspace_items.all().delete()
+
+        # Init secret slug
+        workspace_storage.init_secret_slug()
+        workspace_storage.save()
+
         # Create new workspace items.
         for workspace_edit_item in self.workspace_items.all():
             workspace_storage_item = workspace_edit_item.as_storage(
                 workspace=workspace_storage)
             workspace_storage_item.save()
 
+        # Return slug so it can be shown
+        return workspace_storage.secret_slug
+
     def load_from_storage(self, workspace_storage):
         """Load model and workspace_items from Storage.
         """
@@ -583,6 +431,13 @@ class WorkspaceEdit(
         """Used by wms.html"""
         return reverse("lizard_map_workspace_edit_wms")
 
+    def in_workspace(self, workspace_item_name):
+        """Check if the workspace contains an item with the given
+        workspace_item_name."""
+
+        return (self.workspace_items.filter(name=workspace_item_name)
+                .count()) > 0
+
 
 class WorkspaceEditItem(WorkspaceItemMixin):
     """
@@ -604,7 +459,8 @@ class WorkspaceStorage(BackgroundMapMixin, PeriodMixin, ExtentMixin,
     """
     name = models.CharField(max_length=40)
     description = models.TextField(null=True, blank=True)
-    owner = models.ForeignKey(User)
+    owner = models.ForeignKey(User, null=True, blank=True)
+    secret_slug = models.CharField(max_length=16, null=True)
 
     def __unicode__(self):
         return u'%s (%s)' % (self.name, self.owner)
@@ -614,6 +470,11 @@ class WorkspaceStorage(BackgroundMapMixin, PeriodMixin, ExtentMixin,
         return reverse("lizard_map_workspace_storage_wms",
                        kwargs={'workspace_storage_id': self.id})
 
+    def init_secret_slug(self):
+        if self.secret_slug is None:
+            self.secret_slug = ''.join(random.choice(SECRET_SLUG_CHARS)
+                                       for i in range(SECRET_SLUG_LENGTH))
+
 
 class WorkspaceStorageItem(WorkspaceItemMixin):
     """
@@ -632,7 +493,19 @@ class CollageEdit(UserSessionMixin):
     """
     User selection of map locations.
     """
-    pass
+
+    def data_in_collage(self, adapter_class, adapter_layer_json,
+                        name, identifier):
+        """Check if an item with the same data already exists in the
+        collage."""
+        for item in (self.collage_items
+            .filter(adapter_class=adapter_class)
+            .filter(adapter_layer_json=adapter_layer_json)
+            .filter(name=name)):
+            # Easiest way to compare identifiers?
+            if json.dumps(item.identifier) == json.dumps(identifier):
+                return True
+        return False
 
 
 class StatisticsMixin(models.Model):
@@ -640,12 +513,12 @@ class StatisticsMixin(models.Model):
     Statistics mixin.
     """
     AGGREGATION_PERIOD_CHOICES = (
-        (ALL, _('all')),
-        (YEAR, _('year')),
-        (QUARTER, _('quarter')),
-        (MONTH, _('month')),
-        (WEEK, _('week')),
-        (DAY, _('day')),
+        (dateperiods.ALL, _('all')),
+        (dateperiods.YEAR, _('year')),
+        (dateperiods.QUARTER, _('quarter')),
+        (dateperiods.MONTH, _('month')),
+        (dateperiods.WEEK, _('week')),
+        (dateperiods.DAY, _('day')),
         )
 
     # Boundary value for statistics.
@@ -655,7 +528,7 @@ class StatisticsMixin(models.Model):
     # Restrict_to_month is used to filter the data.
     restrict_to_month = models.IntegerField(blank=True, null=True)
     aggregation_period = models.IntegerField(
-        choices=AGGREGATION_PERIOD_CHOICES, default=ALL)
+        choices=AGGREGATION_PERIOD_CHOICES, default=dateperiods.ALL)
 
     class Meta:
         abstract = True
@@ -665,7 +538,7 @@ class CollageEditItem(WorkspaceItemMixin, StatisticsMixin):
     collage = models.ForeignKey(
         CollageEdit,
         related_name='collage_items')
-    identifier = JSONField(default="")
+    identifier = jsonfield.JSONField()
 
     class Meta:
         ordering = ('name', )
@@ -705,6 +578,30 @@ class CollageEditItem(WorkspaceItemMixin, StatisticsMixin):
             # Adapter is None
             return self.default_grouping_hint
 
+    # The following methods are used from the collage_edit_detail
+    # page and control which things are shown on that page.
+    @property
+    def data_description(self):
+        """Title to show above this bit of data on the collage page."""
+        return self.adapter.collage_detail_data_description(self.identifier)
+
+    @property
+    def collage_detail_edit_action(self):
+        """Which edit action to show on the collage detail page. Currently
+        the only working option in collage_edit_detail is 'graph'. If this
+        data can't be edited on the collage detail page, return None.
+        Calls 'collage_detail_edit_action' in the adapter."""
+        return self.adapter.collage_detail_edit_action(self.identifier)
+
+    @property
+    def collage_detail_show_edit_block(self):
+        return self.adapter.collage_detail_show_edit_block(self.identifier)
+
+    @property
+    def collage_detail_show_statistics_block(self):
+        return self.adapter.collage_detail_show_statistics_block(
+            self.identifier)
+
     def form_initial(self):
         """Initial values from object for CollageItemEditorForm."""
         result = {}
@@ -734,13 +631,13 @@ class CollageEditItem(WorkspaceItemMixin, StatisticsMixin):
             return []
 
         # Calc periods based on aggregation period setting.
-        periods = calc_aggregation_periods(start_date, end_date,
+        periods = dateperiods.calc_aggregation_periods(start_date, end_date,
                                            self.aggregation_period)
         statistics = []
         for period_start_date, period_end_date in periods:
             if not self.restrict_to_month or (
-                self.aggregation_period != MONTH) or (
-                self.aggregation_period == MONTH and
+                self.aggregation_period != dateperiods.MONTH) or (
+                self.aggregation_period == dateperiods.MONTH and
                 self.restrict_to_month == period_start_date.month):
 
                 # Base statistics for each period.
@@ -758,7 +655,7 @@ class CollageEditItem(WorkspaceItemMixin, StatisticsMixin):
                 # Add name.
                 if statistics_row:
                     statistics_row['name'] = self.name
-                    statistics_row['period'] = fancy_period(
+                    statistics_row['period'] = dateperiods.fancy_period(
                         period_start_date, period_end_date,
                         self.aggregation_period)
                     statistics_row['boundary_value'] = self.boundary_value
@@ -802,11 +699,11 @@ class Legend(models.Model):
     max_value = models.FloatField(default=100)
     steps = models.IntegerField(default=10)
 
-    default_color = ColorField()
-    min_color = ColorField()
-    max_color = ColorField()
-    too_low_color = ColorField()
-    too_high_color = ColorField()
+    default_color = fields.ColorField()
+    min_color = fields.ColorField()
+    max_color = fields.ColorField()
+    too_low_color = fields.ColorField()
+    too_high_color = fields.ColorField()
 
     objects = LegendManager()
 
@@ -829,7 +726,7 @@ class Legend(models.Model):
 
     def legend_values(self):
         """Determines legend steps and values. Required by legend_default."""
-        return legend_values(
+        return fields.legend_values(
             self.min_value, self.max_value,
             self.min_color, self.max_color, self.steps)
 
@@ -846,13 +743,13 @@ class Legend(models.Model):
                 elif k == 'steps':
                     self.steps = int(v)
                 elif k == 'min_color':
-                    self.min_color = Color(v)
+                    self.min_color = fields.Color(v)
                 elif k == 'max_color':
-                    self.max_color = Color(v)
+                    self.max_color = fields.Color(v)
                 elif k == 'too_low_color':
-                    self.too_low_color = Color(v)
+                    self.too_low_color = fields.Color(v)
                 elif k == 'too_high_color':
-                    self.too_high_color = Color(v)
+                    self.too_high_color = fields.Color(v)
             except ValueError, e:
                 logger.exception('Could not parse one of the colors')
                 raise WorkspaceItemError(e)
@@ -980,11 +877,13 @@ class BackgroundMap(models.Model):
     LAYER_TYPE_GOOGLE = 1
     LAYER_TYPE_OSM = 2
     LAYER_TYPE_WMS = 3
+    LAYER_TYPE_TMS = 4
 
     LAYER_TYPE_CHOICES = (
         (LAYER_TYPE_GOOGLE, 'GOOGLE'),
         (LAYER_TYPE_OSM, 'OSM'),
         (LAYER_TYPE_WMS, 'WMS'),
+        (LAYER_TYPE_TMS, 'TMS'),
         )
 
     GOOGLE_TYPE_DEFAULT = 1
@@ -1013,11 +912,11 @@ class BackgroundMap(models.Model):
     layer_url = models.CharField(
         max_length=200,
         null=True, blank=True,
-        help_text='Tile url for use with OSM or WMS',
+        help_text='Tile url for use with OSM or WMS or TMS',
         default='http://tile.openstreetmap.nl/tiles/${z}/${x}/${y}.png')
     layer_names = models.TextField(
         null=True, blank=True,
-        help_text='Fill in layer names in case of WMS')
+        help_text='Fill in layer names in case of WMS or TMS')
 
     class Meta:
         ordering = ('index', )
