@@ -38,12 +38,16 @@ urlpatterns = patterns(
     url(r'^myworkspace/load/$',  # L3
         lizard_map.views.WorkspaceLoadView.as_view(),
         name="lizard_map_workspace_load"),
+
     # Partially the same actions as above,
     # you have to put workspace_id in GET parameter here...
-    # TODO: Update to L3
-    # url(r'^workspaceitem/extent/$',
-    #     'lizard_map.views.workspace_item_extent',
-    #     name="lizard_map_workspace_item_extent"),
+    url(r'^workspaceitem/extent/$',
+        'lizard_map.views.workspace_item_extent',
+        name="lizard_map_workspace_item_extent"),
+    # Same for workspace storages
+    url(r'^workspacestorageitem/extent/$',
+        'lizard_map.views.saved_workspace_item_extent',
+        name="lizard_map_workspace_storage_item_extent"),
 
     url(r'^mycollage/$',  # L3
         lizard_map.views.CollageDetailView.as_view(),
@@ -100,13 +104,20 @@ urlpatterns = patterns(
         name="lizard_map.search_coordinates"),  # L3
     url(r'^workspace/(?P<workspace_storage_id>\d+)/search_name/',
         'lizard_map.views.search_coordinates',
-        {'format': 'name'},
+        {'_format': 'name'},
         name="lizard_map.search_name"),  # L3
+    # Get to the workspace page by means of a slug
+    url(r'^workspace/(?P<workspace_storage_slug>\w+)/$',
+        lizard_map.views.WorkspaceStorageView.as_view(),
+        name="lizard_map_workspace_slug_storage"),
 
     # Adapter
     url(r'^adapter/(?P<adapter_class>.*)/image/$',  # L3
         lizard_map.views.AdapterImageView.as_view(),
         name="lizard_map_adapter_image"),
+    url(r'^adapter/(?P<adapter_class>.*)/flot_graph_data/$',  # L3
+        lizard_map.views.AdapterFlotGraphDataView.as_view(),
+        name="lizard_map_adapter_flot_graph_data"),
     url(r'^adapter/(?P<adapter_class>.*)/values/(?P<output_type>.*)/$',  # L3
         lizard_map.views.AdapterValuesView.as_view(),
         name="lizard_map_adapter_values"),
@@ -145,6 +156,13 @@ urlpatterns = patterns(
     # url(r'^snippet_group/(?P<snippet_group_id>\d+)/statistics/csv/',
     #     'lizard_map.views.export_snippet_group_statistics_csv',
     #     name="lizard_map.export_snippet_group_statistics_csv"),
+
+    url(r'^screen/(?P<slug>.*)/$',
+        lizard_map.views.MapIconView.as_view(),
+        name='lizard_ui.icons'),
+    url(r'^$',
+        lizard_map.views.MapIconView.as_view(),
+        name='lizard_ui.icons'),
     )
 
 urlpatterns += debugmode_urlpatterns()
